#include "image_processing.hpp"


// #define RGA_ALIGN(a, b) a


/***********************************************************
* 描述: 找最小值
* 入参:
* 出参:
* 返回:
************************************************************/
static uchar find_min(uchar n1, uchar n2, uchar n3)
{
	return n1 < n2 ? (n1 < n3 ? n1 : n3) : (n2 < n3 ? n2 : n3);
}


/***********************************************************
* 描述: 找最大值
* 入参:
* 出参:
* 返回:
************************************************************/
static uchar find_max(uchar n1, uchar n2, uchar n3)
{
	return n1 > n2 ? (n1 > n3 ? n1 : n3) : (n2 > n3 ? n2 : n3);
}


/***********************************************************
* 描述: 找中值
* 入参:
* 出参:
* 返回:
************************************************************/
static uchar find_med(uchar n1, uchar n2, uchar n3)
{
	uchar temp;

	if (n1 > n2)
	{
		temp = n1;
		n1 = n2;
		n2 = temp;
	}
	if (n2 > n3)
	{
		temp = n2;
		n2 = n3;
		n3 = temp;
	}
	return n1 > n2 ? n1 : n2;
}


/***********************************************************
* 描述: 升序
* 入参:
* 出参:
* 返回:
************************************************************/
static void sort(uchar *n1, uchar *n2, uchar *n3)
{
	uchar temp;

	if (*n1 > *n2)
	{
		temp = *n2;
		*n2 = *n1;
		*n1 = temp;
	}
	if (*n2 > *n3)
	{
		temp = *n3;
		*n3 = *n2;
		*n2 = temp;
	}
	if (*n1 > *n2)
	{
		temp = *n2;
		*n2 = *n1;
		*n1 = temp;
	}
}


/***********************************************************
* 描述: 
* 入参:
* 出参:
* 返回:
************************************************************/
static uchar Median(uchar *kenel, uchar row_input[3], int k)
{
	uchar temp;
	int i, j;
	uchar min, med, max;

	/* bubble sort*/
	for (i = 0; i < 3; i++)
	{
		for (j = 0; j < 3 - i - 1; j++)
		{
			if (row_input[j] > row_input[j + 1])
			{
				temp = row_input[j];
				row_input[j] = row_input[j + 1];
				row_input[j + 1] = temp;
			}
		}
		*(kenel + 3 * (j + 1) + k) = row_input[j + 1];
	}
	*(kenel + k) = row_input[0];

	min = find_min(*(kenel + 6), *(kenel + 7), *(kenel + 8));
	med = find_med(*(kenel + 3), *(kenel + 4), *(kenel + 5));
	max = find_max(*(kenel + 0), *(kenel + 1), *(kenel + 2));
	return find_med(min, med, max);

#if 0
	/* bubble sort*/
	for (int i = 0; i < 9/2 + 1; i++)
	{   
		for (int j = i; j < 9 - 1 - i; j++)
		{ 
			if (arr[j] > arr[j + 1])
			{
				temp = arr[j];
				arr[j] = arr[j + 1];
				arr[j + 1] = temp;
			}
		}
	}
	return arr[4];
#endif
}


/***********************************************************
* 描述: 直方图均衡化
* 入参:
* 出参:
* 返回:
************************************************************/
void histogram_equalization(const Mat src, Mat &dst)
{
	int gray[256] = {0};
	double gray_distribution[256] = {0};
	int gray_equal[256] = {0};
	int gray_sum = src.cols * src.rows;
	Mat yuvImg;

	if (!src.data || !dst.data)
		return;

	cv::cvtColor(src, yuvImg, CV_BGR2YUV_I420);

	for (int i = 0; i < gray_sum; i++)
	{
		gray[yuvImg.data[i]]++;
	}
	gray_distribution[0] = ((double)gray[0] / gray_sum);

	for (int i = 1; i < 256; i++)
	{
		gray_distribution[i] = gray_distribution[i - 1] + (double)gray[i] / gray_sum;
		gray_equal[i] = (uchar)(255 * gray_distribution[i] + 0.5);
	}

	for (int i = 0; i < gray_sum; i++)
	{
		yuvImg.data[i] = gray_equal[yuvImg.data[i]];
	}

	cv::cvtColor(yuvImg, dst, CV_YUV2BGR_I420);
}

/***********************************************************
* 描述: 中值平滑滤波
* 入参:
* 出参:
* 返回:
************************************************************/
void median_smooth_flitering(const Mat src, Mat &dst)
{
	uchar kenel[3][3];
	uchar input_col[3];
	Mat yuvImg;
	int gray_sum = src.cols * src.rows;

	if (!src.data || !dst.data)
		return;

	uchar *pYuvBuf = new unsigned char[gray_sum];
	cv::cvtColor(src, yuvImg, CV_BGR2YUV_I420);
	memcpy(pYuvBuf, yuvImg.data, gray_sum * sizeof(unsigned char));

	for (int i = 0; i < src.rows; ++i)
	{
		if (i > 0 && i < (src.rows - 1))
		{
			kenel[0][0] = pYuvBuf[(i - 1) * src.cols];
			kenel[0][1] = pYuvBuf[(i - 1) * src.cols + 1];
			kenel[0][2] = pYuvBuf[(i - 1) * src.cols + 2];
			kenel[1][0] = pYuvBuf[i * src.cols];
			kenel[1][1] = pYuvBuf[i * src.cols + 1];
			kenel[1][2] = pYuvBuf[i * src.cols + 2];
			kenel[2][0] = pYuvBuf[(i + 1) * src.cols];
			kenel[2][1] = pYuvBuf[(i + 1) * src.cols + 1];
			kenel[2][2] = pYuvBuf[(i + 1) * src.cols + 2];
			sort(&kenel[0][0], &kenel[1][0], &kenel[2][0]);
			sort(&kenel[0][1], &kenel[1][1], &kenel[2][1]);
			sort(&kenel[0][2], &kenel[1][2], &kenel[2][2]);
		}
		for (int j = 0; j < src.cols; ++j)
		{
			if (i > 0 && i < (src.rows - 1) && j > 0 && j < (src.cols - 1))
			{
				input_col[0] = pYuvBuf[(i - 1) * src.cols + j + 1];
				input_col[1] = pYuvBuf[i * src.cols + j + 1];
				input_col[2] = pYuvBuf[(i + 1) * src.cols + j + 1];
				;

				yuvImg.data[i * src.cols + j] = Median(&kenel[0][0], input_col, (j + 1) % 3);
			}
		}
	}
	cv::cvtColor(yuvImg, dst, CV_YUV2BGR_I420);
	delete pYuvBuf;
}


/***********************************************************
* 描述: 边缘检测
* 入参:
* 出参:
* 返回:
************************************************************/
void edge_extraction(const Mat src, Mat &dst)
{
	Mat yuvImg;
	int gray_sum = src.cols * src.rows;

	if (!src.data || !dst.data)
		return;

	uchar *pYuvBuf = new unsigned char[gray_sum];
	cv::cvtColor(src, yuvImg, CV_BGR2YUV_I420);
	memcpy(pYuvBuf, yuvImg.data, gray_sum * sizeof(unsigned char));
	/*sharpen_flitering Template 3x3:
							  0 1 0
							  1 -4 1
							  0 1 0   
	   */
	for (int i = 0; i < src.rows; ++i)
	{
		for (int j = 0; j < src.cols; ++j)
		{
			if (i > 0 && i < (src.rows - 1) && j > 0 && j < (src.cols - 1))
			{
				yuvImg.data[i * src.cols + j] = cv::saturate_cast<uchar>(-4 * pYuvBuf[i * src.cols + j] + pYuvBuf[(i - 1) * src.cols + j] + pYuvBuf[(i + 1) * src.cols + j] + pYuvBuf[i * src.cols + j - 1] + pYuvBuf[i * src.cols + j + 1]);
			}
		}
	}
	cv::cvtColor(yuvImg, dst, CV_YUV2BGR_I420);
	delete pYuvBuf;
}


/***********************************************************
* 描述: 整型滤波器
* 入参:
* 出参:
* 返回:
************************************************************/
void sharpen_flitering(const Mat src, Mat &dst)
{
	Mat yuvImg;
	int gray_sum = src.cols * src.rows;

	if (!src.data || !dst.data)
		return;

	uchar *pYuvBuf = new unsigned char[gray_sum];
	cv::cvtColor(src, yuvImg, CV_BGR2YUV_I420);
	memcpy(pYuvBuf, yuvImg.data, gray_sum * sizeof(unsigned char));
	/*sharpen_flitering Template 3x3:
							  0 -1 0
							 -1 5 -1
							  0 -1 0   
	   */
	for (int i = 0; i < src.rows; ++i)
	{
		for (int j = 0; j < src.cols; ++j)
		{
			if (i > 0 && i < (src.rows - 1) && j > 0 && j < (src.cols - 1))
			{
				yuvImg.data[i * src.cols + j] = cv::saturate_cast<uchar>(5 * pYuvBuf[i * src.cols + j] - pYuvBuf[(i - 1) * src.cols + j] - pYuvBuf[(i + 1) * src.cols + j] - pYuvBuf[i * src.cols + j - 1] - pYuvBuf[i * src.cols + j + 1]);
			}
		}
	}
	cv::cvtColor(yuvImg, dst, CV_YUV2BGR_I420);
	delete pYuvBuf;
}


/***********************************************************
* 描述: 双线性插值
* 入参:
* 出参:
* 返回:
************************************************************/
void bilinear_interpolation(const Mat src, Mat &dst)
{
	if (!src.data || !dst.data)
	{
		return;
	}

	int row = MAGNIFICATION * src.rows, col = MAGNIFICATION * src.cols;
	dst.create(cv::Size(RGA_ALIGN(col, 16), RGA_ALIGN(row, 16)), CV_8UC3);

	for (int i = 0; i < row; ++i)
	{
		float x = (i + 0.5) / MAGNIFICATION - 0.5;
		int fx = (int)x;

		x -= fx;

		short x1 = 1.f - x;
		short x2 = 1 - x1;
		for (int j = 0; j < col; ++j)
		{
			float y = (j + 0.5) / MAGNIFICATION - 0.5;
			int fy = (int)y;
			y -= fy;
			short y1 = 1.f - y;
			short y2 = 1 - y1;
			dst.at<cv::Vec3b>(i, j)[0] = src.at<cv::Vec3b>(fx, fy)[0] * x1 * y1 + src.at<cv::Vec3b>(fx + 1, fy)[0] * x2 * y1 + src.at<cv::Vec3b>(fx, fy + 1)[0] * x1 * y2 + src.at<cv::Vec3b>(fx + 1, fy + 1)[0] * x2 * y2;
			dst.at<cv::Vec3b>(i, j)[1] = src.at<cv::Vec3b>(fx, fy)[1] * x1 * y1 + src.at<cv::Vec3b>(fx + 1, fy)[1] * x2 * y1 + src.at<cv::Vec3b>(fx, fy + 1)[1] * x1 * y2 + src.at<cv::Vec3b>(fx + 1, fy + 1)[1] * x2 * y2;
			dst.at<cv::Vec3b>(i, j)[2] = src.at<cv::Vec3b>(fx, fy)[2] * x1 * y1 + src.at<cv::Vec3b>(fx + 1, fy)[2] * x2 * y1 + src.at<cv::Vec3b>(fx, fy + 1)[2] * x1 * y2 + src.at<cv::Vec3b>(fx + 1, fy + 1)[2] * x2 * y2;
		}
	}
}


/***********************************************************
* 描述: 均值
* 入参:
* 出参:
* 返回:
************************************************************/
static uchar average(const Mat &img, int x_min, int x_max, int y_min, int y_max, int k)
{
	int count = (x_max - x_min + 1) * (y_max - y_min + 1);
	int sum = 0;
	for (int i = x_min; i <= x_max; i++)
	{
		for (int j = y_min; j <= y_max; j++)
		{
			sum += img.at<cv::Vec3b>(i, j)[k];
		}
	}
	return sum / count;
}


/***********************************************************
* 描述: 部分的平均规模
* 入参:
* 出参:
* 返回:
************************************************************/
void scale_part_average(const Mat src, Mat &dst)
{

	if (!src.data || !dst.data)
	{
		return;
	}
	int rows = static_cast<int>(src.rows * REDUCTION);
	int cols = static_cast<int>(src.cols * REDUCTION);
	dst.create(cv::Size(RGA_ALIGN(cols, 16), RGA_ALIGN(rows, 16)), CV_8UC3);

	int lastRow = 0;
	int lastCol = 0;

	for (int i = 0; i < rows; i++)
	{
		int row = static_cast<int>((i + 1) / REDUCTION + 0.5) - 1;

		for (int j = 0; j < cols; j++)
		{
			int col = static_cast<int>((j + 1) / REDUCTION + 0.5) - 1;

			dst.at<cv::Vec3b>(i, j)[0] = average(src, lastRow, row, lastCol, col, 0);
			dst.at<cv::Vec3b>(i, j)[1] = average(src, lastRow, row, lastCol, col, 1);
			dst.at<cv::Vec3b>(i, j)[2] = average(src, lastRow, row, lastCol, col, 2);
			lastCol = col + 1;
		}
		lastCol = 0;
		lastRow = row + 1;
	}
}


/***********************************************************
* 描述: 浮雕
* 入参:
* 出参:
* 返回:
************************************************************/
void relief(const Mat src, Mat &dst)
{
	Mat yuvImg;
	int gray_sum = src.cols * src.rows;

	if (!src.data || !dst.data)
		return;

	uchar *pYuvBuf = new unsigned char[gray_sum];
	cv::cvtColor(src, yuvImg, CV_BGR2YUV_I420);
	memcpy(pYuvBuf, yuvImg.data, gray_sum * sizeof(unsigned char));
	for (int i = 0; i < src.rows; ++i)
	{
		for (int j = 0; j < src.cols; ++j)
		{
			if (i > 0 && i < (src.rows - 1) && j > 0 && j < (src.cols - 1))
			{
				yuvImg.data[i * src.cols + j] = cv::saturate_cast<uchar>(pYuvBuf[(i - 1) * src.cols + j - 1] - pYuvBuf[(i + 1) * src.cols + j + 1] + 128);
			}
		}
	}
	cv::cvtColor(yuvImg, dst, CV_YUV2BGR_I420);
	delete pYuvBuf;
}


/***********************************************************
* 描述: 
* 入参:
* 出参:
* 返回:
************************************************************/
static void rgb_to_gray_to_reverse(const Mat src, Mat &dst1, Mat &dst2)
{
	for (int i = 0; i < src.rows; i++)
	{
		uchar *p = dst1.ptr<uchar>(i);
		uchar *q = dst2.ptr<uchar>(i);
		for (int j = 0; j < src.cols; j++)
		{
			*p = (15 * src.at<cv::Vec3b>(i, j)[0] + 75 * src.at<cv::Vec3b>(i, j)[1] + 38 * src.at<cv::Vec3b>(i, j)[2]) >> 7;
			*q = (255 - *p);
			p++;
			q++;
		}
	}
}


/***********************************************************
* 描述: 素描
* 入参:
* 出参:
* 返回:
************************************************************/
void sketch(const Mat src, Mat &dst)
{
	Mat gray, gray_reverse, gray_end;

	if (!src.data || !dst.data)
		return;

	gray.create(cv::Size(RGA_ALIGN(src.cols, 16), RGA_ALIGN(src.rows, 16)), CV_8UC1);
	gray_reverse.create(cv::Size(RGA_ALIGN(src.cols, 16), RGA_ALIGN(src.rows, 16)), CV_8UC1);
	gray_end.create(cv::Size(RGA_ALIGN(src.cols, 16), RGA_ALIGN(src.rows, 16)), CV_8UC1);

	rgb_to_gray_to_reverse(src, gray, gray_reverse);
	GaussianBlur(gray_reverse, gray_reverse, Size(11, 11), 0);

	for (int i = 0; i < src.rows; i++)
	{
		uchar *p1 = gray.ptr<uchar>(i);
		uchar *p2 = gray_reverse.ptr<uchar>(i);
		uchar *q = gray_end.ptr<uchar>(i);
		for (int j = 0; j < src.cols; j++)
		{
			int tmp1 = p1[j];
			int tmp2 = p2[j];
			q[j] = (uchar)min((tmp1 + (tmp1 * tmp2) / (256 - tmp2)), 255);
		}
	}
	cv::cvtColor(gray_end, dst, CV_GRAY2BGR);
	//gray.release();
	//gray_reverse.release();
	//gray_end.release();
}
