#include "esp.h"
#include "image_processing.hpp"

#include <getopt.h>
#include <stdlib.h>

using namespace cv;
using namespace std;

// 结构体数组定义
const struct scene_processing gscene_processing_arr[SCENE_MAX] = {
	{"histogram-equalization", &histogram_equalization},
	{"smooth-filtering(median)", &median_smooth_flitering},
	{"edge-extraction", &edge_extraction},
	{"sharpening-filtering", &sharpen_flitering},
	{"bilinear-interpolation", &bilinear_interpolation},
	{"scale-Part-Average", &scale_part_average},
	{"relief", &relief},
	{"sketch", &sketch},
};

// 定义命令行选项
static const char *gshort_options = "s:";
// 定义命令行选项
static struct option glong_options[] = {
	{"scene", required_argument, NULL, 's'},
	{NULL, 0, NULL, 0}};

// 用法说明
static void usage(char **argv)
{
	printf(
		"Usage: %s [Options]\n\n"
		"Options:\n"
		"-s, --scene"
		"Scene:\n"
		"0 -- histogram equalization;\n"
		"1 -- smooth-filtering(median);\n"
		"2 -- edge_extraction;\n"
		"3 -- sharpening-filtering;\n"
		"4 -- bilinear-interpolation;\n"
		"5 -- scale-Part-Average;\n"
		"6 -- relief;\n"
		"7 -- sketch;\n",
		argv[0]);
}

int main(int argc, char *argv[])
{
	// openlog("IMAGE-PROCESSING", LOG_CONS | LOG_PERROR, 0);
	/* 平台代码初始化 */
	ESP_INIT_runInit();

	int c = 0, scene = 0;
	const string wintitle = "window";
	Mat image;
	Mat image_dst;
	void (*p_scene_processing)(const Mat src, Mat &dst);

	while ((c = getopt_long(argc, argv, gshort_options, glong_options, NULL)) != -1)
	{
		switch (c)
		{
		case 's':
			scene = atoi(optarg);
			break;
		default:
			usage(argv);
			return 0;
		}
	}

	if (scene < HISTOGRAM_EQUALIZATION || scene >= SCENE_MAX)
	{
		printf("没有此选项, 更改为默认选项\n");
		scene = HISTOGRAM_EQUALIZATION;
	}

	/* Window -- create */
	namedWindow(wintitle);
	moveWindow(wintitle, 720, 480);

	namedWindow(gscene_processing_arr[scene].wintitle_dst, WINDOW_AUTOSIZE);
	p_scene_processing = gscene_processing_arr[scene].scene_processing_fun;

	// !必须先创建这个图片
	image_dst.create(cv::Size(720, 480), CV_8UC3);

	while (1)
	{

		ESP_CAMERA_get(image);

		if (image.empty())
			break;

		// 图像处理
		(*p_scene_processing)(image, image_dst);

		imshow(wintitle, image);
		imshow(gscene_processing_arr[scene].wintitle_dst, image_dst);

		//等待按键
		if (waitKey(30) >= 0)
		{
			break;
		}
	}
}
