#ifndef _ESP_H
#define _ESP_H

/* 按字母顺序排列 */
#include <arpa/inet.h>
#include <assert.h>
#include <curl/curl.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <fstream>
#include <getopt.h>
#include <iostream>
#include <json/json.h>
#include <linux/types.h>
#include <list>
#include <net/if.h>
#include <netinet/in.h>
#include <opencv2/opencv.hpp>
#include <pthread.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "tengine_c_api.h"
#include "tengine_operations.h"
#include <syslog.h>
#include <time.h>
#include <unistd.h>
#include <memory.h>
#include <curl/curl.h>
#include "opencv2/videoio.hpp"
#include "opencv2/core.hpp"
#include "opencv2/highgui.hpp"
#include "opencv2/imgproc.hpp"
extern "C" {
#include <rockchip/rockchip_rga.h>
}
#include <rockchip/rockchip_isp.h>
#include <rockchip/rockchip_mpp.h>
#include "rtspclient.h"

#include <math.h>
#include <vector>
#include <map>
#include <utility>
#include <iomanip>

#include <sys/time.h>


using namespace cv;
using namespace Json;
using namespace std;

#define ESP_MAC_LEN 6
#define ESP_SPEECH_DATA_LEN 2048

typedef enum
{
    ESP_OK,
    ESP_ERROR = -1
}ESP_ERROR_E;

typedef enum
{
    SPEECH_WORD_XIAOZHI = 1,
    SPEECH_WORD_OPEN_CURTAIN = 2, //打开窗帘
    SPEECH_WORD_CLOSE_CURTAIN = 3,
    SPEECH_WORD_OPEN_AIR_CONDITION = 4, //打开空调
    SPEECH_WORD_CLOSE_AIR_CONDITION = 5
}SPEECH_WORD_E;

/* Callback typedef for Tengine2 subscription */
typedef int (*subscribeTopicCallback)(const char*);

/* Model head data from Tengine2 */
typedef struct
{
    int     magicNum;           //魔法值，固定0x3040
    long    headLen;            //数据头长度
    int     type;               //包类型(1:软件包，2:模型)
    int     fileCount;          //文件个数
    char    modelName[64];      //模型名称
    char    modelVersion[64];   //模型版本
    char    deviceTypeCode[64]; //设备型号
    int     dataType;           //数据类型(1:图像,2音频:,3文本:)
    int     dataCategory;       //模型数据分类(101:物体检测,102:人脸检测,103:人脸分类,201:语音唤醒,202:命令词识别)
    int     frameEngine;        //框架引擎(1:TensorFlow,2MXNet:,3:PyTorch)
    char    precisionType[32];  //精度类型(fp32;fp16;int8)
    char    algorithmType[64];  //算法类型
    int     modelId;            //模型ID
    char    reserve[252];       //保留位
    long    bodyLens;           //数据长度(除数据头)
}ModelHeadData;

typedef enum
{
    CAM_OV9750 = 0,
    CAM_IMX258,
    CAMERA_TYPE_USB,
    CAMERA_TYPE_NET,
    CAM_MAX,
}CAM_TYPE;

struct buffer {
    void *start;
    size_t length;
};

class v4l2Camera {

private:
    pthread_mutex_t mutex;
    RockchipRga *rga;
    MppDecoder *dec;
    int fd;

    struct buffer *buffers = NULL;
    int n_buffers;

    __u32 width;
    __u32 height;
    RgaRotate rotate;
    __u32 cropx;
    __u32 cropy;
    __u32 cropw;
    __u32 croph;
    __u32 format;

    int recv_count;

    void* rkengine;
    CAM_TYPE cam_type;

public:
    v4l2Camera(__u32 w, __u32 h, RgaRotate r, int V, int H, __u32 cx, __u32 cy, __u32 cw, __u32 ch, __u32 cam_format);
    ~v4l2Camera();


    unsigned long microTime();
    int init(const char *v4l2_dev, char *isp_dev, CAM_TYPE type);
    int streamOn(void);
    int streamOff(void);
    int readFrame(__u32 dstFormat, Mat& image);
    int enqueue(unsigned char *buf, size_t len);
    DecFrame *dequeue(void);
    void freeFrame(DecFrame *frame);
    void rgaProcess(DecFrame *frame, __u32 dstFormat, Mat& mat);
    void onStreamReceive(unsigned char *buf, size_t len);
};
typedef struct Bbox{
    float x1;
    float y1;
    float x2;
    float y2;
    float score;
    int class_idx;
} Bbox;

/* Compiler Related Definitions */
#if defined(__CC_ARM) || defined(__CLANG_ARM)   /* ARM Compiler */
    #define SECTION(x)                  __attribute__((section(x)))
#elif defined (__GNUC__)                /* GNU GCC Compiler */
    #define SECTION(x)                  __attribute__((section(x)))
#else
    #error not supported tool chain
#endif


typedef ESP_ERROR_E (*ESP_INIT_P) (void);

#define ESP_INIT_EXPORT_START(fn) \
        const ESP_INIT_P __esp_##fn  SECTION(".esp_init_start") = fn
#define ESP_INIT_EXPORT(fn) \
    const ESP_INIT_P __esp_##fn  SECTION(".esp_init") = fn
#define ESP_INIT_EXPORT_END(fn) \
        const ESP_INIT_P __esp_##fn  SECTION(".esp_init_end") = fn


#define ESP_UNINIT_EXPORT_START(fn) \
        const ESP_INIT_P __esp_##fn  SECTION(".esp_uninit_start") = fn
#define ESP_UNINIT_EXPORT(fn) \
    const ESP_INIT_P __esp_##fn  SECTION(".esp_uninit") = fn
#define ESP_UNINIT_EXPORT_END(fn) \
        const ESP_INIT_P __esp_##fn  SECTION(".esp_uninit_end") = fn


ESP_ERROR_E ESP_INIT_runInit();
ESP_ERROR_E ESP_INIT_runUninit();





size_t ESP_COMMON_readFile(const char* filepath, char* buf, int bufLen);
size_t ESP_COMMON_writeFile(const char* filepath, const char* buf, int bufLen);
size_t ESP_COMMON_appendFile(const char* filepath, const void* buf, int bufLen);
ESP_ERROR_E ESP_COMMON_checkFile(const char* filepath);
ESP_ERROR_E ESP_COMMON_getIniConfig(char *title,char *key,char *filename,char *buf);
ESP_ERROR_E ESP_COMMON_execute(const char *fmt, ...);
int ESP_COMMON_getPid(const char *processName);
ESP_ERROR_E ESP_COMMON_getIpByIfname(const char *ifName, struct sockaddr_in *pAddr);
ESP_ERROR_E ESP_COMMON_getJsonValue(const char* file, const char* key, std::vector<std::string>& strVec);
ESP_ERROR_E ESP_COMMON_getJsonValueString(const char* file, const char* key, std::string& strVal);
ESP_ERROR_E ESP_COMMON_setJsonValueString(const char* file, const char* key, const char* strVal);
void ESP_COMMON_md5(const uint8_t *initialMsg, size_t initialLen, uint8_t *digest);
void ESP_COMMON_hexUpperToString(char *hex, int hexLen, char *string);
void ESP_COMMON_hexLowerToString(char *hex, int hexLen, char *string);
const char* ESP_COMMON_getDeviceId();
;
bool ESP_CAMERA_get(cv::Mat& image);
#endif
