#include "esp.h"

#define FRAME_RATE 50  //摄像头采集的帧率

pthread_mutex_t gMutexCamera; 

v4l2Camera* gipc;
Mat gimage;

static void* ThreadNetCamera(void* arg)
{
	DecFrame *frame = NULL;
	while(1)
	{
		frame = gipc->dequeue();
		if(frame != NULL)
		{
			pthread_mutex_lock(&gMutexCamera);
			gipc->rgaProcess(frame, V4L2_PIX_FMT_RGB24, gimage);
			pthread_mutex_unlock(&gMutexCamera);
			usleep(2000);
			gipc->freeFrame(frame);
		}
	usleep(10*1000);
	}
	return 0;
}
__u32 width = 640, height = 480; /* resolution: 640x480 */
static void* ThreadLocalCamera(void* arg)
{
	int ret;
	Mat image;
	image.create(cv::Size(RGA_ALIGN(width, 16), RGA_ALIGN(height, 16)), CV_8UC3);
	while(1)
	{
		pthread_mutex_lock(&gMutexCamera);
		ret = gipc->readFrame(V4L2_PIX_FMT_RGB24, image);
		if(ret == 0)
		{
			gimage = image.clone(); //clone安全点
		}
		pthread_mutex_unlock(&gMutexCamera);

		usleep(1000/FRAME_RATE * 1000);
	}
	return 0;
}

ESP_ERROR_E ESP_CAMERA_init(void)
{
	int ret;
	char v4l2_dev[64], isp_dev[64];
	static __u32 v4l2_type;
	FILE   *stream; 
	char buf[2048];
	size_t size;
	pthread_t threadLocalCamera,threadNetCamera;
	char configFile[]="/home/openailab/ESP/platform/release/config.ini";

	char in_username[64],in_url[64],in_passwd[64];
	ESP_COMMON_getIniConfig((char*)std::string("CAMERA").c_str(),(char*)std::string("in_url").c_str(), configFile,in_url);
	ESP_COMMON_getIniConfig((char*)std::string("CAMERA").c_str(),(char*)std::string("in_passwd").c_str(), configFile,in_passwd);
	ESP_COMMON_getIniConfig((char*)std::string("CAMERA").c_str(),(char*)std::string("in_username").c_str(), configFile,in_username);
	/* MIPI Camera -- default values */
	int camera = 1;    /* main camera */
	bool localCamera = true;
	CAM_TYPE type = CAM_OV9750; /* HD camera sensor: OV9750 */
	//__u32 width = 640, height = 480; /* resolution: 640x480 */
	RgaRotate rotate = RGA_ROTATE_NONE; /* No rotation */
	__u32 cropx = 0, cropy = 0, cropw = 0, croph = 0;
	int vflip = 0, hflip = 0; /* no flip */

	stream = popen("ls -l /sys/class/video4linux/ | awk '/video/{print NR}'", "r" );
	size = fread( buf, 1, sizeof(buf),	stream);  //将刚刚FILE* stream的数据流读取到buf中
	pclose(stream);
	if(size == 0)
	{
		printf("no local camera\n");
		localCamera = false;
	}

	if (localCamera == true)
	{
		stream = popen("ls -l /sys/class/video4linux/ |grep usb |cut -d ':' -f 2|cut -d ' ' -f 2|cut -b 6", "r" );
		size = fread( buf, 1, sizeof(buf),	stream);  //将刚刚FILE* stream的数据流读取到buf中
		pclose(stream);
		if(size)
		{
			printf("find usb camera, open video%d\n",(buf[0] - '0'));
			camera = (buf[0] - '0');
			v4l2_type = V4L2_PIX_FMT_YUYV;
			type = CAMERA_TYPE_USB;
		}
		else
		{
			printf("not find usb camera, use mipi camera\n");
			camera = 2;
			v4l2_type = V4L2_PIX_FMT_NV12;
			type = CAM_OV9750;
		}
		/* V4L2 device */
		sprintf(v4l2_dev, "/dev/video%d", camera);
		sprintf(isp_dev, "/dev/video%d", 1);
	}
	else
	{
		type = CAMERA_TYPE_NET;
	}
	gipc = new v4l2Camera(width, height, rotate, vflip, hflip, cropx, cropy, cropw, croph, v4l2_type);

	gimage.create(cv::Size(RGA_ALIGN(width, 16), RGA_ALIGN(height, 16)), CV_8UC3);
	ret = gipc->init(v4l2_dev, isp_dev, type);
	if(ret < 0)
	{
		printf("v4l2Camera initialization failed.\n");
		return ESP_ERROR;
	}

	if (type == CAMERA_TYPE_NET)
	{
		RtspClient *client = new RtspClient(in_url,in_username,in_passwd);
		ret = pthread_create(&threadNetCamera, NULL, ThreadNetCamera, NULL);
		if(ret < 0)
		{
			printf("pthread_create failed\n");
			return ESP_ERROR;
		}
		client->setDataCallback(std::bind(&v4l2Camera::onStreamReceive, gipc, std::placeholders::_1, std::placeholders::_2));
		client->enable();
	}
	else
	{
		ret = gipc->streamOn();
		if(ret < 0)
		{
			printf("stream on err!\n");
			return ESP_ERROR;
		}
		ret = pthread_create(&threadLocalCamera, NULL, ThreadLocalCamera, NULL);
		if(ret < 0)
		{
			printf("pthread_create failed\n");
			return ESP_ERROR;
		}
	}
	return ESP_OK;
}

bool ESP_CAMERA_get(cv::Mat& image)
{
	pthread_mutex_lock(&gMutexCamera);
	image = gimage.clone();
	pthread_mutex_unlock(&gMutexCamera);
	return true;
}

ESP_INIT_EXPORT(ESP_CAMERA_init);
