#include "esp.h"



size_t ESP_COMMON_readFile(const char* filepath, char* buf, int bufLen)
{
	FILE * pFile;
	size_t size;

	pFile = fopen(filepath, "r");
	if (pFile == NULL) 
	{
		syslog(LOG_USER | LOG_ERR, "[%s(%d)] fopen %s failed(%s)\n", __FUNCTION__, __LINE__, filepath, strerror(errno));
		return 0;
	}

 
	size = fread(buf, 1, bufLen, pFile);
	fclose(pFile);

	return size;
}

size_t ESP_COMMON_writeFile(const char* filepath, const char* buf, int bufLen)
{
	FILE * pFile;
	size_t size;

	pFile = fopen(filepath, "w");
	if (pFile == NULL) 
	{
		syslog(LOG_USER | LOG_ERR, "[%s(%d)] fopen %s failed(%s)\n", __FUNCTION__, __LINE__, filepath, strerror(errno));
		return 0;
	}
 
	size = fwrite(buf, 1, bufLen, pFile);
	fclose(pFile);

	return size;
}



/****************************************************************
 * 描述：将buf中的内容附加到指定文件后
 * 输入：filepath - 文件绝对路径
 *		 buf - 要写的数据
 *		 bufLen - 要写的数据长度
 * 输出：无
 * 返回：写入的字节数，可能为0
 ****************************************************************/
size_t ESP_COMMON_appendFile(const char* filepath, const void* buf, int bufLen)
{
	FILE * pFile;
	size_t size;

	pFile = fopen(filepath, "a");
	if (pFile == NULL) 
	{
		syslog(LOG_USER | LOG_ERR, "[%s(%d)] fopen %s failed(%s)\n", __FUNCTION__, __LINE__, filepath, strerror(errno));
		return 0;
	}
 
	size = fwrite(buf, 1, bufLen, pFile);
	fclose(pFile);

	return size;
}



/****************************************************************
 * 描述：检测文件是否存在并且可读写
 * 输入：filepath - 文件绝对路径
 * 输出：无
 * 返回：成功 - ESP_OK， 失败-ESP_ERROR
 ****************************************************************/
ESP_ERROR_E ESP_COMMON_checkFile(const char* filepath)
{
	int ret;

	ret = access(filepath, R_OK | W_OK);
	if (ret != 0) 
	{
		return ESP_ERROR;
	}


	return ESP_OK;
}


ESP_ERROR_E ESP_COMMON_getIniConfig(char *title,char *key,char *filename,char *buf)  
{  
    FILE *fp;  
    int  flag = 0;  
    char sTitle[64], *wTmp;
    char sLine[1024];        
    sprintf(sTitle, "[%s]", title);
                     
    if(NULL == (fp = fopen(filename, "r"))) {  
        syslog(LOG_USER | LOG_ERR, "[%s(%d)] open %s failed\n", __FUNCTION__, __LINE__, filename);
        return ESP_ERROR;
    }
    while (NULL != fgets(sLine, 1024, fp)) {  
        // 这是注释行  
        if (0 == strncmp("//", sLine, 2)) continue;  
        if ('#' == sLine[0])              continue;        
        wTmp = strchr(sLine, '=');  
        if ((NULL != wTmp) && (1 == flag)) {  
            if (0 == strncmp(key, sLine, strlen(key))) { // 长度依文件读取的为准  
                sLine[strlen(sLine) - 1] = '\0';  
                fclose(fp);
                while(*(wTmp + 1) == ' '){
                    wTmp++;
                }
                strcpy(buf,wTmp + 1);
                return ESP_OK;  
            }  
        } else {  
            if (0 == strncmp(sTitle, sLine, strlen(sTitle))) { // 长度依文件读取的为准  
                flag = 1; // 找到标题位置  
            }  
        }  
    }  
    fclose(fp);  
    return ESP_OK;  
}    


/****************************************************************
 * 描述：执行命令
 * 输入：fmt - 可变参数的字符串
 * 输出：无
 * 返回：ESP_OK
 ****************************************************************/
ESP_ERROR_E ESP_COMMON_execute(const char *fmt, ...)
{
	char cmd[512];
	va_list vargs;

	va_start(vargs, fmt);
	vsnprintf(cmd, sizeof(cmd), fmt, vargs);
	va_end(vargs);
	system(cmd);

	return ESP_OK;
}



/****************************************************************
 * 描述：通过进程名称查询PID
 * 输入：processName - 进程名称
 * 输出：无
 * 返回：进程PID
 ****************************************************************/
int ESP_COMMON_getPid(const char *processName)
{
	#define BUF_SIZE 1024
	DIR *dir;
	struct dirent *ptr;
	FILE *fp;
	char filepath[1024];		//大小随意，能装下cmdline文件的路径即可
	char cur_task_name[50]; //大小随意，能装下要识别的命令行文本即可
	char buf[BUF_SIZE];
	dir = opendir("/proc"); //打开路径
	if (NULL != dir)
	{
		while ((ptr = readdir(dir)) != NULL) //循环读取路径下的每一个文件/文件夹
		{
			//如果读取到的是"."或者".."则跳过，读取到的不是文件夹名字也跳过
			if ((strcmp(ptr->d_name, ".") == 0) || (strcmp(ptr->d_name, "..") == 0))
				continue;
			if (DT_DIR != ptr->d_type)
				continue;

			sprintf(filepath, "/proc/%s/status", ptr->d_name); //生成要读取的文件的路径
			fp = fopen(filepath, "r");						   //打开文件
			if (NULL != fp)
			{
				if (fgets(buf, BUF_SIZE - 1, fp) == NULL)
				{
					continue;
				}
				sscanf(buf, "%*s %s", cur_task_name);
				//如果文件内容满足要求则打印路径的名字（即进程的PID）
				if (!strcmp(processName, cur_task_name))
				{
					fclose(fp);
					closedir(dir);
					return atoi(ptr->d_name);
				}
				fclose(fp);
			}
		}
		closedir(dir); //关闭路径
	}
	return -1;
}

ESP_ERROR_E ESP_COMMON_getIpByIfname(const char *ifName, struct sockaddr_in *pAddr)
{
	struct ifreq ifr;
	int sock;
	int ret;
	
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	
	strncpy(ifr.ifr_name, ifName, sizeof(ifr.ifr_name));
	ret = ioctl(sock, SIOCGIFADDR, &ifr);
	close(sock); //用完立即关闭，避免后面忘记

	if (0 != ret)
	{
		syslog(LOG_USER | LOG_ERR, "[%s(%d)] SIOCGIFADDR failed\n", __FUNCTION__, __LINE__);
		return ESP_ERROR;
	}

	memcpy(pAddr, &(ifr.ifr_addr), sizeof(struct sockaddr_in)) ;

	
	return ESP_OK;
}

/****************************************************************
 * 描述：读取json文件中对应key的字符串值
 * 输入：file json文件名
 * 输入：key 要查找的key。NOTE:仅支持Value类型为字符串或字符串数组!
 * 输出：strVec 查找的结果会追加到strVec
 * 返回：错误码
 ****************************************************************/
ESP_ERROR_E ESP_COMMON_getJsonValue(const char* file, const char* key, std::vector<std::string>& strVec)
{
    std::ifstream in(file, std::ios::binary);
    if (!in.is_open())
    {
        syslog(LOG_USER | LOG_ERR, "[%s(%d)] Open json file error. File name:%s\n", __FUNCTION__, __LINE__, file);
        return ESP_ERROR;
    }

    CharReaderBuilder builder;
    Value root;
    string errs;
    if (!parseFromStream(builder, in, &root, &errs))
    {
        return ESP_ERROR;
    }
    in.close();
    if (root[key].isString())
    {
        strVec.push_back(root[key].asString());
    }
    else if (root[key].isArray())
    {
        for (unsigned int i=0; i<root[key].size(); i++)
        {
            strVec.push_back(root[key][i].asString());
        }
    }
    else
    {
        syslog(LOG_USER | LOG_ERR, "[%s(%d)] Json value type error or key not found.\n", __FUNCTION__, __LINE__);
        return ESP_ERROR;
    }
    return ESP_OK;
}

/****************************************************************
 * 描述：读取json文件中对应key的字符串值
 * 输入：file json文件名
 * 输入：key 要查找的key。NOTE:仅支持Value类型为字符串!
 * 输出：value 查找的字符串结果
 * 返回：错误码
 ****************************************************************/
ESP_ERROR_E ESP_COMMON_getJsonValueString(const char* file, const char* key, std::string& strVal)
{
    std::ifstream in(file, std::ios::binary);
    if (!in.is_open())
    {
        std::cout << "Open json file error.\n";
        return ESP_ERROR;
    }

    CharReaderBuilder builder;
    Value root;
    string errs;
    if (!parseFromStream(builder, in, &root, &errs))
    {
        return ESP_ERROR;
    }
    in.close();
    if (root[key].isString())
    {
        strVal = root[key].asString();
    }
    else
    {
        syslog(LOG_USER | LOG_ERR, "[%s(%d)] Json value type error or key not found.\n", __FUNCTION__, __LINE__);
        return ESP_ERROR;
    }
    return ESP_OK;
}

/****************************************************************
 * 描述：向json文件中写入对应key的字符串值
 * 输入：file json文件名
 * 输入：key 要写入的key。有则覆盖，无则追加
 * 输入：value 要查找的key。NOTE:仅支持Value类型为字符串!
 * 输出：无
 * 返回：错误码
 ****************************************************************/
ESP_ERROR_E ESP_COMMON_setJsonValueString(const char* file, const char* key, const char* strVal)
{
    Value root;
    if (access(file, F_OK) == 0)
    {
        ifstream fin(file);
        if (!fin.is_open())
        {
            syslog(LOG_USER | LOG_ERR, "[%s(%d)] Open json file error. File name:%s\n", __FUNCTION__, __LINE__, file);
            return ESP_ERROR;
        }

        bool parseRet;
        string errs;
        CharReaderBuilder builder;
        parseRet = parseFromStream(builder, fin, &root, &errs);
        fin.close();
        if (!parseRet)
        {
            syslog(LOG_USER | LOG_ERR, "[%s(%d)] Json value type error or key not found.\n", __FUNCTION__, __LINE__);
            return ESP_ERROR;
        }
    }

    root[key] = strVal;

    StreamWriterBuilder writerBuilder;
    StreamWriter* writer = writerBuilder.newStreamWriter();
    ofstream fout(file);
    if(!fout.is_open())
    {
        syslog(LOG_USER | LOG_ERR, "[%s(%d)] Open json file error. File name:%s\n", __FUNCTION__, __LINE__, file);
        return ESP_ERROR;
    }
    writer->write(root, &fout);
    fout.flush();
    fout.close();
    return ESP_OK;
}

/********************************ESP md5 start********************************/
// Constants are the integer part of the sines of integers (in radians) * 2^32.
const uint32_t k[64] = {
0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee ,
0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501 ,
0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be ,
0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821 ,
0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa ,
0xd62f105d, 0x02441453, 0xd8a1e681, 0xe7d3fbc8 ,
0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed ,
0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a ,
0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c ,
0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70 ,
0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x04881d05 ,
0xd9d4d039, 0xe6db99e5, 0x1fa27cf8, 0xc4ac5665 ,
0xf4292244, 0x432aff97, 0xab9423a7, 0xfc93a039 ,
0x655b59c3, 0x8f0ccc92, 0xffeff47d, 0x85845dd1 ,
0x6fa87e4f, 0xfe2ce6e0, 0xa3014314, 0x4e0811a1 ,
0xf7537e82, 0xbd3af235, 0x2ad7d2bb, 0xeb86d391 };

// r specifies the per-round shift amounts
const uint32_t r[] = {7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22, 7, 12, 17, 22,
                      5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20, 5,  9, 14, 20,
                      4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23, 4, 11, 16, 23,
                      6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21, 6, 10, 15, 21};

// leftrotate function definition
#define LEFTROTATE(x, c) (((x) << (c)) | ((x) >> (32 - (c))))

void to_bytes(uint32_t val, uint8_t *bytes)
{
    bytes[0] = (uint8_t) val;
    bytes[1] = (uint8_t) (val >> 8);
    bytes[2] = (uint8_t) (val >> 16);
    bytes[3] = (uint8_t) (val >> 24);
}

uint32_t to_int32(const uint8_t *bytes)
{
    return (uint32_t) bytes[0]
        | ((uint32_t) bytes[1] << 8)
        | ((uint32_t) bytes[2] << 16)
        | ((uint32_t) bytes[3] << 24);
}


/****************************************************************
 * 描述：md5加密
 * 输入：initialMsg 待加密的字符串
 * 输入：initialLen 待加密的字符串
 * 输出：digest 输出的字符串
 * 返回：无
 * 示例：
 *   char encrptStr[256] = {0};
 *   char MD5[128] = {0}; //至少16
 *   md5((uint8_t*)encrptStr, strlen(encrptStr), MD5);
 ****************************************************************/
void ESP_COMMON_md5(const uint8_t *initialMsg, size_t initialLen, uint8_t *digest)
{
    // These vars will contain the hash
    uint32_t h0, h1, h2, h3;

    // Message (to prepare)
    uint8_t *msg = NULL;

    size_t newLen, offset;
    uint32_t w[16];
    uint32_t a, b, c, d, i, f, g, temp;

    // Initialize variables - simple count in nibbles:
    h0 = 0x67452301;
    h1 = 0xefcdab89;
    h2 = 0x98badcfe;
    h3 = 0x10325476;

    //Pre-processing:
    //append "1" bit to message
    //append "0" bits until message length in bits ≡ 448 (mod 512)
    //append length mod (2^64) to message
    for (newLen = initialLen + 1; newLen % (512/8) != 448/8; newLen++)
        ;

    msg = (uint8_t*)malloc(newLen + 8);
    memcpy(msg, initialMsg, initialLen);
    msg[initialLen] = 0x80; // append the "1" bit; most significant bit is "first"
    for (offset = initialLen + 1; offset < newLen; offset++)
        msg[offset] = 0; // append "0" bits

    // append the len in bits at the end of the buffer.
    to_bytes(initialLen*8, msg + newLen);
    // initial_len>>29 == initial_len*8>>32, but avoids overflow.
    to_bytes(initialLen>>29, msg + newLen + 4);

    // Process the message in successive 512-bit chunks:
    //for each 512-bit chunk of message:
    for(offset=0; offset<newLen; offset += (512/8))
    {
        // break chunk into sixteen 32-bit words w[j], 0 ≤ j ≤ 15
        for (i = 0; i < 16; i++)
            w[i] = to_int32(msg + offset + i*4);

        // Initialize hash value for this chunk:
        a = h0;
        b = h1;
        c = h2;
        d = h3;

        // Main loop:
        for(i = 0; i<64; i++)
        {
            if (i < 16) {
                f = (b & c) | ((~b) & d);
                g = i;
            } else if (i < 32) {
                f = (d & b) | ((~d) & c);
                g = (5*i + 1) % 16;
            } else if (i < 48) {
                f = b ^ c ^ d;
                g = (3*i + 5) % 16;
            } else {
                f = c ^ (b | (~d));
                g = (7*i) % 16;
            }
            temp = d;
            d = c;
            c = b;
            b = b + LEFTROTATE((a + f + k[i] + w[g]), r[i]);
            a = temp;
        }
        // Add this chunk's hash to result so far:
        h0 += a;
        h1 += b;
        h2 += c;
        h3 += d;
    }
    // cleanup
    free(msg);

    //var char digest[16] := h0 append h1 append h2 append h3 //(Output is in little-endian)
    to_bytes(h0, digest);
    to_bytes(h1, digest + 4);
    to_bytes(h2, digest + 8);
    to_bytes(h3, digest + 12);
}
/********************************ESP md5 end********************************/


/****************************************************************
 * 描述：十六进制转字符串
 * 输入：hex 待转换的字符
 * 输入：hexLen 待加密的字符串
 * 输出：digest 输出的字符串, NOTE:十六进制为大写字母
 * 返回：无
 * 示例：
 *  #define inputLength 16
 *  char input[inputLength] = {0};
 *  char output[128] = {0}; // 至少 2 * inputLength
    hexToString(input, inputLength, output);
 ****************************************************************/
void ESP_COMMON_hexUpperToString(char *hex, int hexLen, char *string)
{
    int i;
    for(i = 0; i < hexLen; i++)
    {
        sprintf(string + i*2, "%02X", ((unsigned char*)hex)[i]);
    }
}

/****************************************************************
 * 描述：十六进制转字符串
 * 输入：hex 待转换的字符
 * 输入：hexLen 待加密的字符串
 * 输出：digest 输出的字符串, NOTE:十六进制为小写字母
 * 返回：无
 * 示例：
 *  #define inputLength 16
 *  char input[inputLength] = {0};
 *  char output[128] = {0}; // 至少 2 * inputLength
    hexToString(input, inputLength, output);
 ****************************************************************/
void ESP_COMMON_hexLowerToString(char *hex, int hexLen, char *string)
{
    int i;
    for(i = 0; i < hexLen; i++)
    {
        sprintf(string + i*2, "%02x", ((unsigned char*)hex)[i]);
    }
}


/********************************ESP deviceID start********************************/
void getMAC(uint8_t *data)
{
    char buf[64] = {0};
    FILE *fp;
    fp = fopen("/sys/bus/nvmem/devices/rockchip-efuse0/nvmem", "r");
    fread(buf, 32, 1, fp);
    fclose(fp);

    for(int i=0; i<6; i++)
    {
        data[i] = buf[i + 16];
    }
}

void getOtherIDInDevice(uint8_t *data)
{
    char buf[64] ={0};
    FILE *fp;
    fp = fopen("/sys/bus/nvmem/devices/rockchip-efuse0/nvmem", "r");
    fread(buf, 32, 1, fp);
    fclose(fp);

    for(int i=0; i<10; i++)
    {
        data[i] = buf[i + 6];
    }
}

char sChipId[32] = {0};


/****************************************************************
 * 描述：获取设备ID，由Mac地址和cpuinfo组成
 * 输入：无
 * 输出：无
 * 返回：设备ID字符串，示例：00000000050d2154504e3638392e3030
 ****************************************************************/
const char* ESP_COMMON_getDeviceId()
{
    uint8_t id[16];
    getMAC(id);
    getOtherIDInDevice(&id[6]);

    for(int i=0; i<16; i++)
    {
        sprintf(&sChipId[2 * i], "%02x", id[i]);
    }
    return sChipId;
}

/********************************ESP deviceID end********************************/
