#include "esp.h"

#define ESP_VERSION "V1.0.1"


/***********************************************************************************
* 描述: ESP初始化起始函数，没有具体的作用
* 入参: 无
* 出参: 无
* 返回: 无
************************************************************************************/
ESP_ERROR_E startInit()
{
	syslog(LOG_USER | LOG_INFO, "[%s(%d)] ESP init start, version is %s\n", __FUNCTION__, __LINE__, ESP_VERSION);
	return ESP_OK;
}
ESP_INIT_EXPORT_START(startInit);



/***********************************************************************************
* 描述: ESP注销起始函数，没有具体的作用
* 入参: 无
* 出参: 无
* 返回: 无
************************************************************************************/
ESP_ERROR_E startUninit()
{
	syslog(LOG_USER | LOG_INFO, "[%s(%d)] ESP uninit start\n", __FUNCTION__, __LINE__);
	return ESP_OK;
}
ESP_UNINIT_EXPORT_START(startUninit);



/***********************************************************************************
* 描述: ESP初始化结束函数，没有具体的作用
* 入参: 无
* 出参: 无
* 返回: 无
************************************************************************************/
ESP_ERROR_E endInit()
{
	syslog(LOG_USER | LOG_INFO, "[%s(%d)] ESP init end\n", __FUNCTION__, __LINE__);
	return ESP_OK;
}
ESP_INIT_EXPORT_END(endInit);



/***********************************************************************************
* 描述: ESP注销结束函数，没有具体的作用
* 入参: 无
* 出参: 无
* 返回: 无
************************************************************************************/
ESP_ERROR_E endUninit()
{
	syslog(LOG_USER | LOG_INFO, "[%s(%d)] ESP uninit end\n", __FUNCTION__, __LINE__);
	return ESP_OK;
}
ESP_UNINIT_EXPORT_END(endUninit);




/***********************************************************************************
* 描述: 按顺序运行ESP_INIT_EXPORT的函数
* 入参: 无
* 出参: 无
* 返回: 成功-ESP_OK, 失败-ESP_ERROR
************************************************************************************/
ESP_ERROR_E ESP_INIT_runInit()
{
	const ESP_INIT_P *fn_ptr;
	int ret = ESP_OK;

	for (fn_ptr = &__esp_startInit; fn_ptr <= &__esp_endInit; fn_ptr++)
	{
		ret |= (*fn_ptr)();
	}

	return (ESP_ERROR_E)ret;
}



/***********************************************************************************
* 描述: 按顺序运行ESP_UNINIT_EXPORT的函数
* 入参: 无
* 出参: 无
* 返回: 成功-ESP_OK, 失败-ESP_ERROR
************************************************************************************/
ESP_ERROR_E ESP_INIT_runUninit()
{
	const ESP_INIT_P *fn_ptr;
	int ret = ESP_OK;

	for (fn_ptr = &__esp_startUninit; fn_ptr <= &__esp_endUninit; fn_ptr++)
	{
		ret |= (*fn_ptr)();
	}

	return (ESP_ERROR_E)ret;
}


